@@ -114,142 +114,144 @@ With tighter constraints on inputs can squeeze carries into int32.
 //static
 FE_25519 FE_25519::mul(const FE_25519& f, const FE_25519& g)
    {
-   int32_t f0 = f[0];
-   int32_t f1 = f[1];
-   int32_t f2 = f[2];
-   int32_t f3 = f[3];
-   int32_t f4 = f[4];
-   int32_t f5 = f[5];
-   int32_t f6 = f[6];
-   int32_t f7 = f[7];
-   int32_t f8 = f[8];
-   int32_t f9 = f[9];
+   const int32_t f0 = f[0];
+   const int32_t f1 = f[1];
+   const int32_t f2 = f[2];
+   const int32_t f3 = f[3];
+   const int32_t f4 = f[4];
+   const int32_t f5 = f[5];
+   const int32_t f6 = f[6];
+   const int32_t f7 = f[7];
+   const int32_t f8 = f[8];
+   const int32_t f9 = f[9];
 
-   int32_t g0 = g[0];
-   int32_t g1 = g[1];
-   int32_t g2 = g[2];
-   int32_t g3 = g[3];
-   int32_t g4 = g[4];
-   int32_t g5 = g[5];
-   int32_t g6 = g[6];
-   int32_t g7 = g[7];
-   int32_t g8 = g[8];
-   int32_t g9 = g[9];
+   const int32_t g0 = g[0];
+   const int32_t g1 = g[1];
+   const int32_t g2 = g[2];
+   const int32_t g3 = g[3];
+   const int32_t g4 = g[4];
+   const int32_t g5 = g[5];
+   const int32_t g6 = g[6];
+   const int32_t g7 = g[7];
+   const int32_t g8 = g[8];
+   const int32_t g9 = g[9];
+
+   const int32_t g1_19 = 19 * g1; /* 1.959375*2^29 */
+   const int32_t g2_19 = 19 * g2; /* 1.959375*2^30; still ok */
+   const int32_t g3_19 = 19 * g3;
+   const int32_t g4_19 = 19 * g4;
+   const int32_t g5_19 = 19 * g5;
+   const int32_t g6_19 = 19 * g6;
+   const int32_t g7_19 = 19 * g7;
+   const int32_t g8_19 = 19 * g8;
+   const int32_t g9_19 = 19 * g9;
+   const int32_t f1_2 = 2 * f1;
+   const int32_t f3_2 = 2 * f3;
+   const int32_t f5_2 = 2 * f5;
+   const int32_t f7_2 = 2 * f7;
+   const int32_t f9_2 = 2 * f9;
+
+   const int64_t f0g0    = f0   * static_cast<int64_t>(g0);
+   const int64_t f0g1    = f0   * static_cast<int64_t>(g1);
+   const int64_t f0g2    = f0   * static_cast<int64_t>(g2);
+   const int64_t f0g3    = f0   * static_cast<int64_t>(g3);
+   const int64_t f0g4    = f0   * static_cast<int64_t>(g4);
+   const int64_t f0g5    = f0   * static_cast<int64_t>(g5);
+   const int64_t f0g6    = f0   * static_cast<int64_t>(g6);
+   const int64_t f0g7    = f0   * static_cast<int64_t>(g7);
+   const int64_t f0g8    = f0   * static_cast<int64_t>(g8);
+   const int64_t f0g9    = f0   * static_cast<int64_t>(g9);
+   const int64_t f1g0    = f1   * static_cast<int64_t>(g0);
+   const int64_t f1g1_2  = f1_2 * static_cast<int64_t>(g1);
+   const int64_t f1g2    = f1   * static_cast<int64_t>(g2);
+   const int64_t f1g3_2  = f1_2 * static_cast<int64_t>(g3);
+   const int64_t f1g4    = f1   * static_cast<int64_t>(g4);
+   const int64_t f1g5_2  = f1_2 * static_cast<int64_t>(g5);
+   const int64_t f1g6    = f1   * static_cast<int64_t>(g6);
+   const int64_t f1g7_2  = f1_2 * static_cast<int64_t>(g7);
+   const int64_t f1g8    = f1   * static_cast<int64_t>(g8);
+   const int64_t f1g9_38 = f1_2 * static_cast<int64_t>(g9_19);
+   const int64_t f2g0    = f2   * static_cast<int64_t>(g0);
+   const int64_t f2g1    = f2   * static_cast<int64_t>(g1);
+   const int64_t f2g2    = f2   * static_cast<int64_t>(g2);
+   const int64_t f2g3    = f2   * static_cast<int64_t>(g3);
+   const int64_t f2g4    = f2   * static_cast<int64_t>(g4);
+   const int64_t f2g5    = f2   * static_cast<int64_t>(g5);
+   const int64_t f2g6    = f2   * static_cast<int64_t>(g6);
+   const int64_t f2g7    = f2   * static_cast<int64_t>(g7);
+   const int64_t f2g8_19 = f2   * static_cast<int64_t>(g8_19);
+   const int64_t f2g9_19 = f2   * static_cast<int64_t>(g9_19);
+   const int64_t f3g0    = f3   * static_cast<int64_t>(g0);
+   const int64_t f3g1_2  = f3_2 * static_cast<int64_t>(g1);
+   const int64_t f3g2    = f3   * static_cast<int64_t>(g2);
+   const int64_t f3g3_2  = f3_2 * static_cast<int64_t>(g3);
+   const int64_t f3g4    = f3   * static_cast<int64_t>(g4);
+   const int64_t f3g5_2  = f3_2 * static_cast<int64_t>(g5);
+   const int64_t f3g6    = f3   * static_cast<int64_t>(g6);
+   const int64_t f3g7_38 = f3_2 * static_cast<int64_t>(g7_19);
+   const int64_t f3g8_19 = f3   * static_cast<int64_t>(g8_19);
+   const int64_t f3g9_38 = f3_2 * static_cast<int64_t>(g9_19);
+   const int64_t f4g0    = f4   * static_cast<int64_t>(g0);
+   const int64_t f4g1    = f4   * static_cast<int64_t>(g1);
+   const int64_t f4g2    = f4   * static_cast<int64_t>(g2);
+   const int64_t f4g3    = f4   * static_cast<int64_t>(g3);
+   const int64_t f4g4    = f4   * static_cast<int64_t>(g4);
+   const int64_t f4g5    = f4   * static_cast<int64_t>(g5);
+   const int64_t f4g6_19 = f4   * static_cast<int64_t>(g6_19);
+   const int64_t f4g7_19 = f4   * static_cast<int64_t>(g7_19);
+   const int64_t f4g8_19 = f4   * static_cast<int64_t>(g8_19);
+   const int64_t f4g9_19 = f4   * static_cast<int64_t>(g9_19);
+   const int64_t f5g0    = f5   * static_cast<int64_t>(g0);
+   const int64_t f5g1_2  = f5_2 * static_cast<int64_t>(g1);
+   const int64_t f5g2    = f5   * static_cast<int64_t>(g2);
+   const int64_t f5g3_2  = f5_2 * static_cast<int64_t>(g3);
+   const int64_t f5g4    = f5   * static_cast<int64_t>(g4);
+   const int64_t f5g5_38 = f5_2 * static_cast<int64_t>(g5_19);
+   const int64_t f5g6_19 = f5   * static_cast<int64_t>(g6_19);
+   const int64_t f5g7_38 = f5_2 * static_cast<int64_t>(g7_19);
+   const int64_t f5g8_19 = f5   * static_cast<int64_t>(g8_19);
+   const int64_t f5g9_38 = f5_2 * static_cast<int64_t>(g9_19);
+   const int64_t f6g0    = f6   * static_cast<int64_t>(g0);
+   const int64_t f6g1    = f6   * static_cast<int64_t>(g1);
+   const int64_t f6g2    = f6   * static_cast<int64_t>(g2);
+   const int64_t f6g3    = f6   * static_cast<int64_t>(g3);
+   const int64_t f6g4_19 = f6   * static_cast<int64_t>(g4_19);
+   const int64_t f6g5_19 = f6   * static_cast<int64_t>(g5_19);
+   const int64_t f6g6_19 = f6   * static_cast<int64_t>(g6_19);
+   const int64_t f6g7_19 = f6   * static_cast<int64_t>(g7_19);
+   const int64_t f6g8_19 = f6   * static_cast<int64_t>(g8_19);
+   const int64_t f6g9_19 = f6   * static_cast<int64_t>(g9_19);
+   const int64_t f7g0    = f7   * static_cast<int64_t>(g0);
+   const int64_t f7g1_2  = f7_2 * static_cast<int64_t>(g1);
+   const int64_t f7g2    = f7   * static_cast<int64_t>(g2);
+   const int64_t f7g3_38 = f7_2 * static_cast<int64_t>(g3_19);
+   const int64_t f7g4_19 = f7   * static_cast<int64_t>(g4_19);
+   const int64_t f7g5_38 = f7_2 * static_cast<int64_t>(g5_19);
+   const int64_t f7g6_19 = f7   * static_cast<int64_t>(g6_19);
+   const int64_t f7g7_38 = f7_2 * static_cast<int64_t>(g7_19);
+   const int64_t f7g8_19 = f7   * static_cast<int64_t>(g8_19);
+   const int64_t f7g9_38 = f7_2 * static_cast<int64_t>(g9_19);
+   const int64_t f8g0    = f8   * static_cast<int64_t>(g0);
+   const int64_t f8g1    = f8   * static_cast<int64_t>(g1);
+   const int64_t f8g2_19 = f8   * static_cast<int64_t>(g2_19);
+   const int64_t f8g3_19 = f8   * static_cast<int64_t>(g3_19);
+   const int64_t f8g4_19 = f8   * static_cast<int64_t>(g4_19);
+   const int64_t f8g5_19 = f8   * static_cast<int64_t>(g5_19);
+   const int64_t f8g6_19 = f8   * static_cast<int64_t>(g6_19);
+   const int64_t f8g7_19 = f8   * static_cast<int64_t>(g7_19);
+   const int64_t f8g8_19 = f8   * static_cast<int64_t>(g8_19);
+   const int64_t f8g9_19 = f8   * static_cast<int64_t>(g9_19);
+   const int64_t f9g0    = f9   * static_cast<int64_t>(g0);
+   const int64_t f9g1_38 = f9_2 * static_cast<int64_t>(g1_19);
+   const int64_t f9g2_19 = f9   * static_cast<int64_t>(g2_19);
+   const int64_t f9g3_38 = f9_2 * static_cast<int64_t>(g3_19);
+   const int64_t f9g4_19 = f9   * static_cast<int64_t>(g4_19);
+   const int64_t f9g5_38 = f9_2 * static_cast<int64_t>(g5_19);
+   const int64_t f9g6_19 = f9   * static_cast<int64_t>(g6_19);
+   const int64_t f9g7_38 = f9_2 * static_cast<int64_t>(g7_19);
+   const int64_t f9g8_19 = f9   * static_cast<int64_t>(g8_19);
+   const int64_t f9g9_38 = f9_2 * static_cast<int64_t>(g9_19);
 
-   int32_t g1_19 = 19 * g1; /* 1.959375*2^29 */
-   int32_t g2_19 = 19 * g2; /* 1.959375*2^30; still ok */
-   int32_t g3_19 = 19 * g3;
-   int32_t g4_19 = 19 * g4;
-   int32_t g5_19 = 19 * g5;
-   int32_t g6_19 = 19 * g6;
-   int32_t g7_19 = 19 * g7;
-   int32_t g8_19 = 19 * g8;
-   int32_t g9_19 = 19 * g9;
-   int32_t f1_2 = 2 * f1;
-   int32_t f3_2 = 2 * f3;
-   int32_t f5_2 = 2 * f5;
-   int32_t f7_2 = 2 * f7;
-   int32_t f9_2 = 2 * f9;
-   int64_t f0g0    = f0   * static_cast<int64_t>(g0);
-   int64_t f0g1    = f0   * static_cast<int64_t>(g1);
-   int64_t f0g2    = f0   * static_cast<int64_t>(g2);
-   int64_t f0g3    = f0   * static_cast<int64_t>(g3);
-   int64_t f0g4    = f0   * static_cast<int64_t>(g4);
-   int64_t f0g5    = f0   * static_cast<int64_t>(g5);
-   int64_t f0g6    = f0   * static_cast<int64_t>(g6);
-   int64_t f0g7    = f0   * static_cast<int64_t>(g7);
-   int64_t f0g8    = f0   * static_cast<int64_t>(g8);
-   int64_t f0g9    = f0   * static_cast<int64_t>(g9);
-   int64_t f1g0    = f1   * static_cast<int64_t>(g0);
-   int64_t f1g1_2  = f1_2 * static_cast<int64_t>(g1);
-   int64_t f1g2    = f1   * static_cast<int64_t>(g2);
-   int64_t f1g3_2  = f1_2 * static_cast<int64_t>(g3);
-   int64_t f1g4    = f1   * static_cast<int64_t>(g4);
-   int64_t f1g5_2  = f1_2 * static_cast<int64_t>(g5);
-   int64_t f1g6    = f1   * static_cast<int64_t>(g6);
-   int64_t f1g7_2  = f1_2 * static_cast<int64_t>(g7);
-   int64_t f1g8    = f1   * static_cast<int64_t>(g8);
-   int64_t f1g9_38 = f1_2 * static_cast<int64_t>(g9_19);
-   int64_t f2g0    = f2   * static_cast<int64_t>(g0);
-   int64_t f2g1    = f2   * static_cast<int64_t>(g1);
-   int64_t f2g2    = f2   * static_cast<int64_t>(g2);
-   int64_t f2g3    = f2   * static_cast<int64_t>(g3);
-   int64_t f2g4    = f2   * static_cast<int64_t>(g4);
-   int64_t f2g5    = f2   * static_cast<int64_t>(g5);
-   int64_t f2g6    = f2   * static_cast<int64_t>(g6);
-   int64_t f2g7    = f2   * static_cast<int64_t>(g7);
-   int64_t f2g8_19 = f2   * static_cast<int64_t>(g8_19);
-   int64_t f2g9_19 = f2   * static_cast<int64_t>(g9_19);
-   int64_t f3g0    = f3   * static_cast<int64_t>(g0);
-   int64_t f3g1_2  = f3_2 * static_cast<int64_t>(g1);
-   int64_t f3g2    = f3   * static_cast<int64_t>(g2);
-   int64_t f3g3_2  = f3_2 * static_cast<int64_t>(g3);
-   int64_t f3g4    = f3   * static_cast<int64_t>(g4);
-   int64_t f3g5_2  = f3_2 * static_cast<int64_t>(g5);
-   int64_t f3g6    = f3   * static_cast<int64_t>(g6);
-   int64_t f3g7_38 = f3_2 * static_cast<int64_t>(g7_19);
-   int64_t f3g8_19 = f3   * static_cast<int64_t>(g8_19);
-   int64_t f3g9_38 = f3_2 * static_cast<int64_t>(g9_19);
-   int64_t f4g0    = f4   * static_cast<int64_t>(g0);
-   int64_t f4g1    = f4   * static_cast<int64_t>(g1);
-   int64_t f4g2    = f4   * static_cast<int64_t>(g2);
-   int64_t f4g3    = f4   * static_cast<int64_t>(g3);
-   int64_t f4g4    = f4   * static_cast<int64_t>(g4);
-   int64_t f4g5    = f4   * static_cast<int64_t>(g5);
-   int64_t f4g6_19 = f4   * static_cast<int64_t>(g6_19);
-   int64_t f4g7_19 = f4   * static_cast<int64_t>(g7_19);
-   int64_t f4g8_19 = f4   * static_cast<int64_t>(g8_19);
-   int64_t f4g9_19 = f4   * static_cast<int64_t>(g9_19);
-   int64_t f5g0    = f5   * static_cast<int64_t>(g0);
-   int64_t f5g1_2  = f5_2 * static_cast<int64_t>(g1);
-   int64_t f5g2    = f5   * static_cast<int64_t>(g2);
-   int64_t f5g3_2  = f5_2 * static_cast<int64_t>(g3);
-   int64_t f5g4    = f5   * static_cast<int64_t>(g4);
-   int64_t f5g5_38 = f5_2 * static_cast<int64_t>(g5_19);
-   int64_t f5g6_19 = f5   * static_cast<int64_t>(g6_19);
-   int64_t f5g7_38 = f5_2 * static_cast<int64_t>(g7_19);
-   int64_t f5g8_19 = f5   * static_cast<int64_t>(g8_19);
-   int64_t f5g9_38 = f5_2 * static_cast<int64_t>(g9_19);
-   int64_t f6g0    = f6   * static_cast<int64_t>(g0);
-   int64_t f6g1    = f6   * static_cast<int64_t>(g1);
-   int64_t f6g2    = f6   * static_cast<int64_t>(g2);
-   int64_t f6g3    = f6   * static_cast<int64_t>(g3);
-   int64_t f6g4_19 = f6   * static_cast<int64_t>(g4_19);
-   int64_t f6g5_19 = f6   * static_cast<int64_t>(g5_19);
-   int64_t f6g6_19 = f6   * static_cast<int64_t>(g6_19);
-   int64_t f6g7_19 = f6   * static_cast<int64_t>(g7_19);
-   int64_t f6g8_19 = f6   * static_cast<int64_t>(g8_19);
-   int64_t f6g9_19 = f6   * static_cast<int64_t>(g9_19);
-   int64_t f7g0    = f7   * static_cast<int64_t>(g0);
-   int64_t f7g1_2  = f7_2 * static_cast<int64_t>(g1);
-   int64_t f7g2    = f7   * static_cast<int64_t>(g2);
-   int64_t f7g3_38 = f7_2 * static_cast<int64_t>(g3_19);
-   int64_t f7g4_19 = f7   * static_cast<int64_t>(g4_19);
-   int64_t f7g5_38 = f7_2 * static_cast<int64_t>(g5_19);
-   int64_t f7g6_19 = f7   * static_cast<int64_t>(g6_19);
-   int64_t f7g7_38 = f7_2 * static_cast<int64_t>(g7_19);
-   int64_t f7g8_19 = f7   * static_cast<int64_t>(g8_19);
-   int64_t f7g9_38 = f7_2 * static_cast<int64_t>(g9_19);
-   int64_t f8g0    = f8   * static_cast<int64_t>(g0);
-   int64_t f8g1    = f8   * static_cast<int64_t>(g1);
-   int64_t f8g2_19 = f8   * static_cast<int64_t>(g2_19);
-   int64_t f8g3_19 = f8   * static_cast<int64_t>(g3_19);
-   int64_t f8g4_19 = f8   * static_cast<int64_t>(g4_19);
-   int64_t f8g5_19 = f8   * static_cast<int64_t>(g5_19);
-   int64_t f8g6_19 = f8   * static_cast<int64_t>(g6_19);
-   int64_t f8g7_19 = f8   * static_cast<int64_t>(g7_19);
-   int64_t f8g8_19 = f8   * static_cast<int64_t>(g8_19);
-   int64_t f8g9_19 = f8   * static_cast<int64_t>(g9_19);
-   int64_t f9g0    = f9   * static_cast<int64_t>(g0);
-   int64_t f9g1_38 = f9_2 * static_cast<int64_t>(g1_19);
-   int64_t f9g2_19 = f9   * static_cast<int64_t>(g2_19);
-   int64_t f9g3_38 = f9_2 * static_cast<int64_t>(g3_19);
-   int64_t f9g4_19 = f9   * static_cast<int64_t>(g4_19);
-   int64_t f9g5_38 = f9_2 * static_cast<int64_t>(g5_19);
-   int64_t f9g6_19 = f9   * static_cast<int64_t>(g6_19);
-   int64_t f9g7_38 = f9_2 * static_cast<int64_t>(g7_19);
-   int64_t f9g8_19 = f9   * static_cast<int64_t>(g8_19);
-   int64_t f9g9_38 = f9_2 * static_cast<int64_t>(g9_19);
    int64_t h0 = f0g0+f1g9_38+f2g8_19+f3g7_38+f4g6_19+f5g5_38+f6g4_19+f7g3_38+f8g2_19+f9g1_38;
    int64_t h1 = f0g1+f1g0   +f2g9_19+f3g8_19+f4g7_19+f5g6_19+f6g5_19+f7g4_19+f8g3_19+f9g2_19;
    int64_t h2 = f0g2+f1g1_2 +f2g0   +f3g9_38+f4g8_19+f5g7_38+f6g6_19+f7g5_38+f8g4_19+f9g3_38;
@@ -260,16 +262,6 @@ FE_25519 FE_25519::mul(const FE_25519& f, const FE_25519& g)
    int64_t h7 = f0g7+f1g6   +f2g5   +f3g4   +f4g3   +f5g2   +f6g1   +f7g0   +f8g9_19+f9g8_19;
    int64_t h8 = f0g8+f1g7_2 +f2g6   +f3g5_2 +f4g4   +f5g3_2 +f6g2   +f7g1_2 +f8g0   +f9g9_38;
    int64_t h9 = f0g9+f1g8   +f2g7   +f3g6   +f4g5   +f5g4   +f6g3   +f7g2   +f8g1   +f9g0   ;
-   int64_t carry0;
-   int64_t carry1;
-   int64_t carry2;
-   int64_t carry3;
-   int64_t carry4;
-   int64_t carry5;
-   int64_t carry6;
-   int64_t carry7;
-   int64_t carry8;
-   int64_t carry9;
 
    /*
    |h0| <= (1.65*1.65*2^52*(1+19+19+19+19)+1.65*1.65*2^50*(38+38+38+38+38))
@@ -277,74 +269,49 @@ FE_25519 FE_25519::mul(const FE_25519& f, const FE_25519& g)
    |h1| <= (1.65*1.65*2^51*(1+1+19+19+19+19+19+19+19+19))
    i.e. |h1| <= 1.7*2^59; narrower ranges for h3, h5, h7, h9
    */
-   const int64_t X24 = (1 << 24);
-   const int64_t X25 = (1 << 25);
-   const int64_t X26 = (1 << 26);
+   carry<26>(h0, h1);
+   carry<26>(h4, h5);
 
-   carry0 = (h0 + X25) >> 26;
-   h1 += carry0;
-   h0 -= carry0 * X26;
-   carry4 = (h4 + X25) >> 26;
-   h5 += carry4;
-   h4 -= carry4 * X26;
    /* |h0| <= 2^25 */
    /* |h4| <= 2^25 */
    /* |h1| <= 1.71*2^59 */
    /* |h5| <= 1.71*2^59 */
 
-   carry1 = (h1 + X24) >> 25;
-   h2 += carry1;
-   h1 -= carry1 * X25;
-   carry5 = (h5 + X24) >> 25;
-   h6 += carry5;
-   h5 -= carry5 * X25;
+   carry<25>(h1, h2);
+   carry<25>(h5, h6);
+
    /* |h1| <= 2^24; from now on fits into int32 */
    /* |h5| <= 2^24; from now on fits into int32 */
    /* |h2| <= 1.41*2^60 */
    /* |h6| <= 1.41*2^60 */
 
-   carry2 = (h2 + X25) >> 26;
-   h3 += carry2;
-   h2 -= carry2 * X26;
-   carry6 = (h6 + X25) >> 26;
-   h7 += carry6;
-   h6 -= carry6 * X26;
+   carry<26>(h2, h3);
+   carry<26>(h6, h7);
    /* |h2| <= 2^25; from now on fits into int32 unchanged */
    /* |h6| <= 2^25; from now on fits into int32 unchanged */
    /* |h3| <= 1.71*2^59 */
    /* |h7| <= 1.71*2^59 */
 
-   carry3 = (h3 + X24) >> 25;
-   h4 += carry3;
-   h3 -= carry3 * X25;
-   carry7 = (h7 + X24) >> 25;
-   h8 += carry7;
-   h7 -= carry7 * X25;
+   carry<25>(h3, h4);
+   carry<25>(h7, h8);
    /* |h3| <= 2^24; from now on fits into int32 unchanged */
    /* |h7| <= 2^24; from now on fits into int32 unchanged */
    /* |h4| <= 1.72*2^34 */
    /* |h8| <= 1.41*2^60 */
 
-   carry4 = (h4 + X25) >> 26;
-   h5 += carry4;
-   h4 -= carry4 * X26;
-   carry8 = (h8 + X25) >> 26;
-   h9 += carry8;
-   h8 -= carry8 * X26;
+   carry<26>(h4, h5);
+   carry<26>(h8, h9);
    /* |h4| <= 2^25; from now on fits into int32 unchanged */
    /* |h8| <= 2^25; from now on fits into int32 unchanged */
    /* |h5| <= 1.01*2^24 */
    /* |h9| <= 1.71*2^59 */
 
-   carry9 = (h9 + X24) >> 25;
-   h0 += carry9 * 19;
-   h9 -= carry9 * X25;
+   carry<25, 19>(h9, h0);
+
    /* |h9| <= 2^24; from now on fits into int32 unchanged */
    /* |h0| <= 1.1*2^39 */
 
-   carry0 = (h0 + X25) >> 26;
-   h1 += carry0;
-   h0 -= carry0 * X26;
+   carry<26>(h0, h1);
    /* |h0| <= 2^25; from now on fits into int32 unchanged */
    /* |h1| <= 1.01*2^24 */
 
@@ -369,10 +336,6 @@ See fe_mul.c for discussion of implementation strategy.
 //static
 FE_25519 FE_25519::sqr_iter(const FE_25519& f, size_t iter)
    {
-   const int64_t X24 = (1 << 24);
-   const int64_t X25 = (1 << 25);
-   const int64_t X26 = (1 << 26);
-
    int32_t f0 = f[0];
    int32_t f1 = f[1];
    int32_t f2 = f[2];
@@ -467,58 +430,20 @@ FE_25519 FE_25519::sqr_iter(const FE_25519& f, size_t iter)
       int64_t h8 = f0f8_2+f1f7_4 +f2f6_2 +f3f5_4 +f4f4   +f9f9_38;
       int64_t h9 = f0f9_2+f1f8_2 +f2f7_2 +f3f6_2 +f4f5_2;
 
-      int64_t carry0;
-      int64_t carry1;
-      int64_t carry2;
-      int64_t carry3;
-      int64_t carry4;
-      int64_t carry5;
-      int64_t carry6;
-      int64_t carry7;
-      int64_t carry8;
-      int64_t carry9;
+      carry<26>(h0, h1);
+      carry<26>(h4, h5);
+      carry<25>(h1, h2);
+      carry<25>(h5, h6);
+      carry<26>(h2, h3);
+      carry<26>(h6, h7);
 
-      carry0 = (h0 + X25) >> 26;
-      h1 += carry0;
-      h0 -= carry0 * X26;
-      carry4 = (h4 + X25) >> 26;
-      h5 += carry4;
-      h4 -= carry4 * X26;
+      carry<25>(h3, h4);
+      carry<25>(h7, h8);
 
-      carry1 = (h1 + X24) >> 25;
-      h2 += carry1;
-      h1 -= carry1 * X25;
-      carry5 = (h5 + X24) >> 25;
-      h6 += carry5;
-      h5 -= carry5 * X25;
-
-      carry2 = (h2 + X25) >> 26;
-      h3 += carry2;
-      h2 -= carry2 * X26;
-      carry6 = (h6 + X25) >> 26;
-      h7 += carry6;
-      h6 -= carry6 * X26;
-
-      carry3 = (h3 + X24) >> 25;
-      h4 += carry3;
-      h3 -= carry3 * X25;
-      carry7 = (h7 + X24) >> 25;
-      h8 += carry7;
-      h7 -= carry7 * X25;
-
-      carry4 = (h4 + X25) >> 26;
-      h5 += carry4;
-      h4 -= carry4 * X26;
-      carry8 = (h8 + X25) >> 26;
-      h9 += carry8;
-      h8 -= carry8 * X26;
-
-      carry9 = (h9 + X24) >> 25;
-      h0 += carry9 * 19;
-      h9 -= carry9 * X25;
-      carry0 = (h0 + X25) >> 26;
-      h1 += carry0;
-      h0 -= carry0 * X26;
+      carry<26>(h4, h5);
+      carry<26>(h8, h9);
+      carry<25,19>(h9, h0);
+      carry<26>(h0, h1);
 
       f0 = h0;
       f1 = h1;
@@ -553,88 +478,85 @@ See fe_mul.c for discussion of implementation strategy.
 //static
 FE_25519 FE_25519::sqr2(const FE_25519& f)
    {
-   const int64_t X24 = (1 << 24);
-   const int64_t X25 = (1 << 25);
-   const int64_t X26 = (1 << 26);
+   const int32_t f0 = f[0];
+   const int32_t f1 = f[1];
+   const int32_t f2 = f[2];
+   const int32_t f3 = f[3];
+   const int32_t f4 = f[4];
+   const int32_t f5 = f[5];
+   const int32_t f6 = f[6];
+   const int32_t f7 = f[7];
+   const int32_t f8 = f[8];
+   const int32_t f9 = f[9];
+   const int32_t f0_2 = 2 * f0;
+   const int32_t f1_2 = 2 * f1;
+   const int32_t f2_2 = 2 * f2;
+   const int32_t f3_2 = 2 * f3;
+   const int32_t f4_2 = 2 * f4;
+   const int32_t f5_2 = 2 * f5;
+   const int32_t f6_2 = 2 * f6;
+   const int32_t f7_2 = 2 * f7;
+   const int32_t f5_38 = 38 * f5; /* 1.959375*2^30 */
+   const int32_t f6_19 = 19 * f6; /* 1.959375*2^30 */
+   const int32_t f7_38 = 38 * f7; /* 1.959375*2^30 */
+   const int32_t f8_19 = 19 * f8; /* 1.959375*2^30 */
+   const int32_t f9_38 = 38 * f9; /* 1.959375*2^30 */
+   const int64_t f0f0    = f0   * static_cast<int64_t>(f0);
+   const int64_t f0f1_2  = f0_2 * static_cast<int64_t>(f1);
+   const int64_t f0f2_2  = f0_2 * static_cast<int64_t>(f2);
+   const int64_t f0f3_2  = f0_2 * static_cast<int64_t>(f3);
+   const int64_t f0f4_2  = f0_2 * static_cast<int64_t>(f4);
+   const int64_t f0f5_2  = f0_2 * static_cast<int64_t>(f5);
+   const int64_t f0f6_2  = f0_2 * static_cast<int64_t>(f6);
+   const int64_t f0f7_2  = f0_2 * static_cast<int64_t>(f7);
+   const int64_t f0f8_2  = f0_2 * static_cast<int64_t>(f8);
+   const int64_t f0f9_2  = f0_2 * static_cast<int64_t>(f9);
+   const int64_t f1f1_2  = f1_2 * static_cast<int64_t>(f1);
+   const int64_t f1f2_2  = f1_2 * static_cast<int64_t>(f2);
+   const int64_t f1f3_4  = f1_2 * static_cast<int64_t>(f3_2);
+   const int64_t f1f4_2  = f1_2 * static_cast<int64_t>(f4);
+   const int64_t f1f5_4  = f1_2 * static_cast<int64_t>(f5_2);
+   const int64_t f1f6_2  = f1_2 * static_cast<int64_t>(f6);
+   const int64_t f1f7_4  = f1_2 * static_cast<int64_t>(f7_2);
+   const int64_t f1f8_2  = f1_2 * static_cast<int64_t>(f8);
+   const int64_t f1f9_76 = f1_2 * static_cast<int64_t>(f9_38);
+   const int64_t f2f2    = f2   * static_cast<int64_t>(f2);
+   const int64_t f2f3_2  = f2_2 * static_cast<int64_t>(f3);
+   const int64_t f2f4_2  = f2_2 * static_cast<int64_t>(f4);
+   const int64_t f2f5_2  = f2_2 * static_cast<int64_t>(f5);
+   const int64_t f2f6_2  = f2_2 * static_cast<int64_t>(f6);
+   const int64_t f2f7_2  = f2_2 * static_cast<int64_t>(f7);
+   const int64_t f2f8_38 = f2_2 * static_cast<int64_t>(f8_19);
+   const int64_t f2f9_38 = f2   * static_cast<int64_t>(f9_38);
+   const int64_t f3f3_2  = f3_2 * static_cast<int64_t>(f3);
+   const int64_t f3f4_2  = f3_2 * static_cast<int64_t>(f4);
+   const int64_t f3f5_4  = f3_2 * static_cast<int64_t>(f5_2);
+   const int64_t f3f6_2  = f3_2 * static_cast<int64_t>(f6);
+   const int64_t f3f7_76 = f3_2 * static_cast<int64_t>(f7_38);
+   const int64_t f3f8_38 = f3_2 * static_cast<int64_t>(f8_19);
+   const int64_t f3f9_76 = f3_2 * static_cast<int64_t>(f9_38);
+   const int64_t f4f4    = f4   * static_cast<int64_t>(f4);
+   const int64_t f4f5_2  = f4_2 * static_cast<int64_t>(f5);
+   const int64_t f4f6_38 = f4_2 * static_cast<int64_t>(f6_19);
+   const int64_t f4f7_38 = f4   * static_cast<int64_t>(f7_38);
+   const int64_t f4f8_38 = f4_2 * static_cast<int64_t>(f8_19);
+   const int64_t f4f9_38 = f4   * static_cast<int64_t>(f9_38);
+   const int64_t f5f5_38 = f5   * static_cast<int64_t>(f5_38);
+   const int64_t f5f6_38 = f5_2 * static_cast<int64_t>(f6_19);
+   const int64_t f5f7_76 = f5_2 * static_cast<int64_t>(f7_38);
+   const int64_t f5f8_38 = f5_2 * static_cast<int64_t>(f8_19);
+   const int64_t f5f9_76 = f5_2 * static_cast<int64_t>(f9_38);
+   const int64_t f6f6_19 = f6   * static_cast<int64_t>(f6_19);
+   const int64_t f6f7_38 = f6   * static_cast<int64_t>(f7_38);
+   const int64_t f6f8_38 = f6_2 * static_cast<int64_t>(f8_19);
+   const int64_t f6f9_38 = f6   * static_cast<int64_t>(f9_38);
+   const int64_t f7f7_38 = f7   * static_cast<int64_t>(f7_38);
+   const int64_t f7f8_38 = f7_2 * static_cast<int64_t>(f8_19);
+   const int64_t f7f9_76 = f7_2 * static_cast<int64_t>(f9_38);
+   const int64_t f8f8_19 = f8   * static_cast<int64_t>(f8_19);
+   const int64_t f8f9_38 = f8   * static_cast<int64_t>(f9_38);
+   const int64_t f9f9_38 = f9   * static_cast<int64_t>(f9_38);
 
-   int32_t f0 = f[0];
-   int32_t f1 = f[1];
-   int32_t f2 = f[2];
-   int32_t f3 = f[3];
-   int32_t f4 = f[4];
-   int32_t f5 = f[5];
-   int32_t f6 = f[6];
-   int32_t f7 = f[7];
-   int32_t f8 = f[8];
-   int32_t f9 = f[9];
-   int32_t f0_2 = 2 * f0;
-   int32_t f1_2 = 2 * f1;
-   int32_t f2_2 = 2 * f2;
-   int32_t f3_2 = 2 * f3;
-   int32_t f4_2 = 2 * f4;
-   int32_t f5_2 = 2 * f5;
-   int32_t f6_2 = 2 * f6;
-   int32_t f7_2 = 2 * f7;
-   int32_t f5_38 = 38 * f5; /* 1.959375*2^30 */
-   int32_t f6_19 = 19 * f6; /* 1.959375*2^30 */
-   int32_t f7_38 = 38 * f7; /* 1.959375*2^30 */
-   int32_t f8_19 = 19 * f8; /* 1.959375*2^30 */
-   int32_t f9_38 = 38 * f9; /* 1.959375*2^30 */
-   int64_t f0f0    = f0   * static_cast<int64_t>(f0);
-   int64_t f0f1_2  = f0_2 * static_cast<int64_t>(f1);
-   int64_t f0f2_2  = f0_2 * static_cast<int64_t>(f2);
-   int64_t f0f3_2  = f0_2 * static_cast<int64_t>(f3);
-   int64_t f0f4_2  = f0_2 * static_cast<int64_t>(f4);
-   int64_t f0f5_2  = f0_2 * static_cast<int64_t>(f5);
-   int64_t f0f6_2  = f0_2 * static_cast<int64_t>(f6);
-   int64_t f0f7_2  = f0_2 * static_cast<int64_t>(f7);
-   int64_t f0f8_2  = f0_2 * static_cast<int64_t>(f8);
-   int64_t f0f9_2  = f0_2 * static_cast<int64_t>(f9);
-   int64_t f1f1_2  = f1_2 * static_cast<int64_t>(f1);
-   int64_t f1f2_2  = f1_2 * static_cast<int64_t>(f2);
-   int64_t f1f3_4  = f1_2 * static_cast<int64_t>(f3_2);
-   int64_t f1f4_2  = f1_2 * static_cast<int64_t>(f4);
-   int64_t f1f5_4  = f1_2 * static_cast<int64_t>(f5_2);
-   int64_t f1f6_2  = f1_2 * static_cast<int64_t>(f6);
-   int64_t f1f7_4  = f1_2 * static_cast<int64_t>(f7_2);
-   int64_t f1f8_2  = f1_2 * static_cast<int64_t>(f8);
-   int64_t f1f9_76 = f1_2 * static_cast<int64_t>(f9_38);
-   int64_t f2f2    = f2   * static_cast<int64_t>(f2);
-   int64_t f2f3_2  = f2_2 * static_cast<int64_t>(f3);
-   int64_t f2f4_2  = f2_2 * static_cast<int64_t>(f4);
-   int64_t f2f5_2  = f2_2 * static_cast<int64_t>(f5);
-   int64_t f2f6_2  = f2_2 * static_cast<int64_t>(f6);
-   int64_t f2f7_2  = f2_2 * static_cast<int64_t>(f7);
-   int64_t f2f8_38 = f2_2 * static_cast<int64_t>(f8_19);
-   int64_t f2f9_38 = f2   * static_cast<int64_t>(f9_38);
-   int64_t f3f3_2  = f3_2 * static_cast<int64_t>(f3);
-   int64_t f3f4_2  = f3_2 * static_cast<int64_t>(f4);
-   int64_t f3f5_4  = f3_2 * static_cast<int64_t>(f5_2);
-   int64_t f3f6_2  = f3_2 * static_cast<int64_t>(f6);
-   int64_t f3f7_76 = f3_2 * static_cast<int64_t>(f7_38);
-   int64_t f3f8_38 = f3_2 * static_cast<int64_t>(f8_19);
-   int64_t f3f9_76 = f3_2 * static_cast<int64_t>(f9_38);
-   int64_t f4f4    = f4   * static_cast<int64_t>(f4);
-   int64_t f4f5_2  = f4_2 * static_cast<int64_t>(f5);
-   int64_t f4f6_38 = f4_2 * static_cast<int64_t>(f6_19);
-   int64_t f4f7_38 = f4   * static_cast<int64_t>(f7_38);
-   int64_t f4f8_38 = f4_2 * static_cast<int64_t>(f8_19);
-   int64_t f4f9_38 = f4   * static_cast<int64_t>(f9_38);
-   int64_t f5f5_38 = f5   * static_cast<int64_t>(f5_38);
-   int64_t f5f6_38 = f5_2 * static_cast<int64_t>(f6_19);
-   int64_t f5f7_76 = f5_2 * static_cast<int64_t>(f7_38);
-   int64_t f5f8_38 = f5_2 * static_cast<int64_t>(f8_19);
-   int64_t f5f9_76 = f5_2 * static_cast<int64_t>(f9_38);
-   int64_t f6f6_19 = f6   * static_cast<int64_t>(f6_19);
-   int64_t f6f7_38 = f6   * static_cast<int64_t>(f7_38);
-   int64_t f6f8_38 = f6_2 * static_cast<int64_t>(f8_19);
-   int64_t f6f9_38 = f6   * static_cast<int64_t>(f9_38);
-   int64_t f7f7_38 = f7   * static_cast<int64_t>(f7_38);
-   int64_t f7f8_38 = f7_2 * static_cast<int64_t>(f8_19);
-   int64_t f7f9_76 = f7_2 * static_cast<int64_t>(f9_38);
-   int64_t f8f8_19 = f8   * static_cast<int64_t>(f8_19);
-   int64_t f8f9_38 = f8   * static_cast<int64_t>(f9_38);
-   int64_t f9f9_38 = f9   * static_cast<int64_t>(f9_38);
    int64_t h0 = f0f0  +f1f9_76+f2f8_38+f3f7_76+f4f6_38+f5f5_38;
    int64_t h1 = f0f1_2+f2f9_38+f3f8_38+f4f7_38+f5f6_38;
    int64_t h2 = f0f2_2+f1f1_2 +f3f9_76+f4f8_38+f5f7_76+f6f6_19;
@@ -645,16 +567,6 @@ FE_25519 FE_25519::sqr2(const FE_25519& f)
    int64_t h7 = f0f7_2+f1f6_2 +f2f5_2 +f3f4_2 +f8f9_38;
    int64_t h8 = f0f8_2+f1f7_4 +f2f6_2 +f3f5_4 +f4f4   +f9f9_38;
    int64_t h9 = f0f9_2+f1f8_2 +f2f7_2 +f3f6_2 +f4f5_2;
-   int64_t carry0;
-   int64_t carry1;
-   int64_t carry2;
-   int64_t carry3;
-   int64_t carry4;
-   int64_t carry5;
-   int64_t carry6;
-   int64_t carry7;
-   int64_t carry8;
-   int64_t carry9;
 
    h0 += h0;
    h1 += h1;
@@ -667,48 +579,21 @@ FE_25519 FE_25519::sqr2(const FE_25519& f)
    h8 += h8;
    h9 += h9;
 
-   carry0 = (h0 + X25) >> 26;
-   h1 += carry0;
-   h0 -= carry0 * X26;
-   carry4 = (h4 + X25) >> 26;
-   h5 += carry4;
-   h4 -= carry4 * X26;
+   carry<26>(h0, h1);
+   carry<26>(h4, h5);
 
-   carry1 = (h1 + X24) >> 25;
-   h2 += carry1;
-   h1 -= carry1 * X25;
-   carry5 = (h5 + X24) >> 25;
-   h6 += carry5;
-   h5 -= carry5 * X25;
+   carry<25>(h1, h2);
+   carry<25>(h5, h6);
 
-   carry2 = (h2 + X25) >> 26;
-   h3 += carry2;
-   h2 -= carry2 * X26;
-   carry6 = (h6 + X25) >> 26;
-   h7 += carry6;
-   h6 -= carry6 * X26;
+   carry<26>(h2, h3);
+   carry<26>(h6, h7);
 
-   carry3 = (h3 + X24) >> 25;
-   h4 += carry3;
-   h3 -= carry3 * X25;
-   carry7 = (h7 + X24) >> 25;
-   h8 += carry7;
-   h7 -= carry7 * X25;
-
-   carry4 = (h4 + X25) >> 26;
-   h5 += carry4;
-   h4 -= carry4 * X26;
-   carry8 = (h8 + X25) >> 26;
-   h9 += carry8;
-   h8 -= carry8 * X26;
-
-   carry9 = (h9 + X24) >> 25;
-   h0 += carry9 * 19;
-   h9 -= carry9 * X25;
-
-   carry0 = (h0 + X25) >> 26;
-   h1 += carry0;
-   h0 -= carry0 * X26;
+   carry<25>(h3, h4);
+   carry<25>(h7, h8);
+   carry<26>(h4, h5);
+   carry<26>(h8, h9);
+   carry<25,19>(h9, h0);
+   carry<26>(h0, h1);
 
    return FE_25519(h0, h1, h2, h3, h4, h5, h6, h7, h8, h9);
    }
@@ -719,10 +604,6 @@ Ignores top bit of h.
 
 void FE_25519::from_bytes(const uint8_t s[32])
    {
-   const int64_t X24 = (1 << 24);
-   const int64_t X25 = (1 << 25);
-   const int64_t X26 = (1 << 26);
-
    int64_t h0 = load_4(s);
    int64_t h1 = load_3(s + 4) << 6;
    int64_t h2 = load_3(s + 7) << 5;
@@ -734,37 +615,17 @@ void FE_25519::from_bytes(const uint8_t s[32])
    int64_t h8 = load_3(s + 26) << 4;
    int64_t h9 = (load_3(s + 29) & 0x7fffff) << 2;
 
-   const int64_t carry9 = (h9 + X24) >> 25;
-   h0 += carry9 * 19;
-   h9 -= carry9 * X25;
-   const int64_t carry1 = (h1 + X24) >> 25;
-   h2 += carry1;
-   h1 -= carry1 * X25;
-   const int64_t carry3 = (h3 + X24) >> 25;
-   h4 += carry3;
-   h3 -= carry3 * X25;
-   const int64_t carry5 = (h5 + X24) >> 25;
-   h6 += carry5;
-   h5 -= carry5 * X25;
-   const int64_t carry7 = (h7 + X24) >> 25;
-   h8 += carry7;
-   h7 -= carry7 * X25;
+   carry<25,19>(h9, h0);
+   carry<25>(h1, h2);
+   carry<25>(h3, h4);
+   carry<25>(h5, h6);
+   carry<25>(h7, h8);
 
-   const int64_t carry0 = (h0 + X25) >> 26;
-   h1 += carry0;
-   h0 -= carry0 * X26;
-   const int64_t carry2 = (h2 + X25) >> 26;
-   h3 += carry2;
-   h2 -= carry2 * X26;
-   const int64_t carry4 = (h4 + X25) >> 26;
-   h5 += carry4;
-   h4 -= carry4 * X26;
-   const int64_t carry6 = (h6 + X25) >> 26;
-   h7 += carry6;
-   h6 -= carry6 * X26;
-   const int64_t carry8 = (h8 + X25) >> 26;
-   h9 += carry8;
-   h8 -= carry8 * X26;
+   carry<26>(h0, h1);
+   carry<26>(h2, h3);
+   carry<26>(h4, h5);
+   carry<26>(h6, h7);
+   carry<26>(h8, h9);
 
    m_fe[0] = h0;
    m_fe[1] = h1;
@@ -806,7 +667,6 @@ so floor(2^(-255)(h + 19 2^(-25) h9 + 2^(-1))) = q.
 void FE_25519::to_bytes(uint8_t s[32]) const
    {
    const int64_t X25 = (1 << 25);
-   const int64_t X26 = (1 << 26);
 
    int32_t h0 = m_fe[0];
    int32_t h1 = m_fe[1];
@@ -819,16 +679,6 @@ void FE_25519::to_bytes(uint8_t s[32]) const
    int32_t h8 = m_fe[8];
    int32_t h9 = m_fe[9];
    int32_t q;
-   int32_t carry0;
-   int32_t carry1;
-   int32_t carry2;
-   int32_t carry3;
-   int32_t carry4;
-   int32_t carry5;
-   int32_t carry6;
-   int32_t carry7;
-   int32_t carry8;
-   int32_t carry9;
 
    q = (19 * h9 + ((static_cast<int32_t>(1) << 24))) >> 25;
    q = (h0 + q) >> 26;
@@ -846,34 +696,17 @@ void FE_25519::to_bytes(uint8_t s[32]) const
    h0 += 19 * q;
    /* Goal: Output h-2^255 q, which is between 0 and 2^255-20. */
 
-   carry0 = h0 >> 26;
-   h1 += carry0;
-   h0 -= carry0 * X26;
-   carry1 = h1 >> 25;
-   h2 += carry1;
-   h1 -= carry1 * X25;
-   carry2 = h2 >> 26;
-   h3 += carry2;
-   h2 -= carry2 * X26;
-   carry3 = h3 >> 25;
-   h4 += carry3;
-   h3 -= carry3 * X25;
-   carry4 = h4 >> 26;
-   h5 += carry4;
-   h4 -= carry4 * X26;
-   carry5 = h5 >> 25;
-   h6 += carry5;
-   h5 -= carry5 * X25;
-   carry6 = h6 >> 26;
-   h7 += carry6;
-   h6 -= carry6 * X26;
-   carry7 = h7 >> 25;
-   h8 += carry7;
-   h7 -= carry7 * X25;
-   carry8 = h8 >> 26;
-   h9 += carry8;
-   h8 -= carry8 * X26;
-   carry9 = h9 >> 25;
+   carry0<26>(h0, h1);
+   carry0<25>(h1, h2);
+   carry0<26>(h2, h3);
+   carry0<25>(h3, h4);
+   carry0<26>(h4, h5);
+   carry0<25>(h5, h6);
+   carry0<26>(h6, h7);
+   carry0<25>(h7, h8);
+   carry0<26>(h8, h9);
+
+   int32_t carry9 = h9 >> 25;
    h9 -= carry9 * X25;
    /* h10 = carry9 */
 
