@@ -200,7 +200,7 @@ void Channel::change_cipher_spec_reader(Connection_Side side)
    BOTAN_ASSERT(pending && pending->server_hello(),
                 "Have received server hello");
 
-   if(pending->server_hello()->compression_method() != NO_COMPRESSION)
+   if(pending->server_hello()->compression_method() != 0)
       throw Internal_Error("Negotiated unknown compression algorithm");
 
    sequence_numbers().new_read_cipher_state();
@@ -229,7 +229,7 @@ void Channel::change_cipher_spec_writer(Connection_Side side)
    BOTAN_ASSERT(pending && pending->server_hello(),
                 "Have received server hello");
 
-   if(pending->server_hello()->compression_method() != NO_COMPRESSION)
+   if(pending->server_hello()->compression_method() != 0)
       throw Internal_Error("Negotiated unknown compression algorithm");
 
    sequence_numbers().new_write_cipher_state();
@@ -314,8 +314,7 @@ size_t Channel::received_data(const uint8_t input[], size_t input_size)
                         raw_input,
                         record,
                         m_sequence_numbers.get(),
-                        std::bind(&TLS::Channel::read_cipher_state_epoch, this,
-                                  std::placeholders::_1));
+                        [this](uint16_t epoch) { return read_cipher_state_epoch(epoch); });
 
          BOTAN_ASSERT(consumed > 0, "Got to eat something");
 
