@@ -161,7 +161,7 @@ void Client::send_client_hello(Handshake_State& state_base,
          Ensure that the session protocol type matches what we want to use
          If not skip the resume and establish a new session
          */
-         if(version == session_info.version())
+         if(version == session_info.version() && policy().acceptable_ciphersuite(session_info.ciphersuite()))
             {
             if(srp_identifier == "" || session_info.srp_identifier() == srp_identifier)
                {
@@ -169,6 +169,7 @@ void Client::send_client_hello(Handshake_State& state_base,
                   new Client_Hello(state.handshake_io(),
                                    state.hash(),
                                    policy(),
+                                   callbacks(),
                                    rng(),
                                    secure_renegotiation_data_for_client_hello(),
                                    session_info,
@@ -188,6 +189,7 @@ void Client::send_client_hello(Handshake_State& state_base,
          state.handshake_io(),
          state.hash(),
          policy(),
+         callbacks(),
          rng(),
          secure_renegotiation_data_for_client_hello(),
          client_settings,
@@ -261,11 +263,10 @@ void Client::process_handshake_msg(const Handshake_State* active_state,
                              "Server replied with a signaling ciphersuite");
          }
 
-      if(!value_exists(state.client_hello()->compression_methods(),
-                       state.server_hello()->compression_method()))
+      if(state.server_hello()->compression_method() != 0)
          {
          throw TLS_Exception(Alert::HANDSHAKE_FAILURE,
-                             "Server replied with compression method we didn't send");
+                             "Server replied with non-null compression method");
          }
 
       auto client_extn = state.client_hello()->extension_types();
@@ -295,6 +296,8 @@ void Client::process_handshake_msg(const Handshake_State* active_state,
                                 "Server replied with DTLS-SRTP alg we did not send");
          }
 
+      callbacks().tls_examine_extensions(state.server_hello()->extensions(), SERVER);
+
       state.set_version(state.server_hello()->version());
       m_application_protocol = state.server_hello()->next_protocol();
 
@@ -358,11 +361,11 @@ void Client::process_handshake_msg(const Handshake_State* active_state,
                                 " is unacceptable by policy");
             }
 
-         if(state.ciphersuite().sig_algo() != "")
+         if(state.ciphersuite().signature_used() || state.ciphersuite().kex_method() == Kex_Algo::STATIC_RSA)
             {
             state.set_expected_next(CERTIFICATE);
             }
-         else if(state.ciphersuite().kex_algo() == "PSK")
+         else if(state.ciphersuite().kex_method() == Kex_Algo::PSK)
             {
             /* PSK is anonymous so no certificate/cert req message is
                ever sent. The server may or may not send a server kex,
@@ -375,7 +378,7 @@ void Client::process_handshake_msg(const Handshake_State* active_state,
             state.set_expected_next(SERVER_KEX);
             state.set_expected_next(SERVER_HELLO_DONE);
             }
-         else if(state.ciphersuite().kex_algo() != "RSA")
+         else if(state.ciphersuite().kex_method() != Kex_Algo::STATIC_RSA)
             {
             state.set_expected_next(SERVER_KEX);
             }
@@ -405,13 +408,16 @@ void Client::process_handshake_msg(const Handshake_State* active_state,
 
       std::unique_ptr<Public_Key> peer_key(server_certs[0].subject_public_key());
 
-      if(peer_key->algo_name() != state.ciphersuite().sig_algo())
+      const std::string expected_key_type =
+         state.ciphersuite().signature_used() ? state.ciphersuite().sig_algo() : "RSA";
+
+      if(peer_key->algo_name() != expected_key_type)
          throw TLS_Exception(Alert::ILLEGAL_PARAMETER,
                              "Certificate key type did not match ciphersuite");
 
       state.server_public_key.reset(peer_key.release());
 
-      if(state.ciphersuite().kex_algo() != "RSA")
+      if(state.ciphersuite().kex_method() != Kex_Algo::STATIC_RSA)
          {
          state.set_expected_next(SERVER_KEX);
          }
@@ -448,7 +454,7 @@ void Client::process_handshake_msg(const Handshake_State* active_state,
       {
       state.server_cert_status(new Certificate_Status(contents));
 
-      if(state.ciphersuite().kex_algo() != "RSA")
+      if(state.ciphersuite().kex_method() != Kex_Algo::STATIC_RSA)
          {
          state.set_expected_next(SERVER_KEX);
          }
@@ -465,12 +471,12 @@ void Client::process_handshake_msg(const Handshake_State* active_state,
 
       state.server_kex(
          new Server_Key_Exchange(contents,
-                                 state.ciphersuite().kex_algo(),
-                                 state.ciphersuite().sig_algo(),
+                                 state.ciphersuite().kex_method(),
+                                 state.ciphersuite().auth_method(),
                                  state.version())
          );
 
-      if(state.ciphersuite().sig_algo() != "")
+      if(state.ciphersuite().signature_used())
          {
          const Public_Key& server_key = state.get_server_public_key();
 
@@ -519,9 +525,10 @@ void Client::process_handshake_msg(const Handshake_State* active_state,
          const auto& types = state.cert_req()->acceptable_cert_types();
 
          std::vector<X509_Certificate> client_certs =
-            m_creds.cert_chain(types,
-                               "tls-client",
-                               m_info.hostname());
+            m_creds.find_cert_chain(types,
+                                    state.cert_req()->acceptable_CAs(),
+                                    "tls-client",
+                                    m_info.hostname());
 
          state.client_certs(new Certificate(state.handshake_io(),
                                             state.hash(),
@@ -609,7 +616,6 @@ void Client::process_handshake_msg(const Handshake_State* active_state,
          state.session_keys().master_secret(),
          state.server_hello()->version(),
          state.server_hello()->ciphersuite(),
-         state.server_hello()->compression_method(),
          CLIENT,
          state.server_hello()->supports_extended_master_secret(),
          state.server_hello()->supports_encrypt_then_mac(),
