@@ -32,6 +32,7 @@ struct X509_Certificate_Data
    X509_Time m_not_before;
    X509_Time m_not_after;
    std::vector<uint8_t> m_subject_public_key_bits;
+   std::vector<uint8_t> m_subject_public_key_bits_seq;
    std::vector<uint8_t> m_subject_public_key_bitstring;
    std::vector<uint8_t> m_subject_public_key_bitstring_sha1;
    AlgorithmIdentifier m_subject_public_key_algid;
@@ -85,6 +86,12 @@ X509_Certificate::X509_Certificate(const std::vector<uint8_t>& vec)
    load_data(src);
    }
 
+X509_Certificate::X509_Certificate(const uint8_t data[], size_t len)
+   {
+   DataSource_Memory src(data, len);
+   load_data(src);
+   }
+
 #if defined(BOTAN_TARGET_OS_HAS_FILESYSTEM)
 X509_Certificate::X509_Certificate(const std::string& fsname)
    {
@@ -128,13 +135,11 @@ std::unique_ptr<X509_Certificate_Data> parse_x509_cert_body(const X509_Object& o
    data->m_issuer_dn_bits = ASN1::put_in_sequence(data->m_issuer_dn.get_bits());
 
    BER_Object public_key = tbs_cert.get_next_object();
-   if(public_key.type_tag != SEQUENCE || public_key.class_tag != CONSTRUCTED)
-      throw BER_Bad_Tag("X509_Certificate: Unexpected tag for public key",
-                        public_key.type_tag, public_key.class_tag);
+   public_key.assert_is_a(SEQUENCE, CONSTRUCTED, "X.509 certificate public key");
 
    // validate_public_key_params(public_key.value);
    AlgorithmIdentifier public_key_alg_id;
-   BER_Decoder(public_key.value).decode(public_key_alg_id).discard_remaining();
+   BER_Decoder(public_key).decode(public_key_alg_id).discard_remaining();
 
    std::vector<std::string> public_key_info =
       split_on(OIDS::oid2str(public_key_alg_id.get_oid()), '/');
@@ -180,7 +185,9 @@ std::unique_ptr<X509_Certificate_Data> parse_x509_cert_body(const X509_Object& o
          }
       }
 
-   data->m_subject_public_key_bits = unlock(public_key.value);
+   data->m_subject_public_key_bits.assign(public_key.bits(), public_key.bits() + public_key.length());
+
+   data->m_subject_public_key_bits_seq = ASN1::put_in_sequence(data->m_subject_public_key_bits);
 
    BER_Decoder(data->m_subject_public_key_bits)
       .decode(data->m_subject_public_key_algid)
@@ -190,14 +197,12 @@ std::unique_ptr<X509_Certificate_Data> parse_x509_cert_body(const X509_Object& o
    tbs_cert.decode_optional_string(data->m_v2_subject_key_id, BIT_STRING, 2);
 
    BER_Object v3_exts_data = tbs_cert.get_next_object();
-   if(v3_exts_data.type_tag == 3 &&
-      v3_exts_data.class_tag == ASN1_Tag(CONSTRUCTED | CONTEXT_SPECIFIC))
+   if(v3_exts_data.is_a(3, ASN1_Tag(CONSTRUCTED | CONTEXT_SPECIFIC)))
       {
-      BER_Decoder(v3_exts_data.value).decode(data->m_v3_extensions).verify_end();
+      BER_Decoder(v3_exts_data).decode(data->m_v3_extensions).verify_end();
       }
-   else if(v3_exts_data.type_tag != NO_OBJECT)
-      throw BER_Bad_Tag("Unknown tag in X.509 cert",
-                        v3_exts_data.type_tag, v3_exts_data.class_tag);
+   else if(v3_exts_data.is_set())
+      throw BER_Bad_Tag("Unknown tag in X.509 cert", v3_exts_data.tagging());
 
    if(tbs_cert.more_items())
       throw Decoding_Error("TBSCertificate has extra data after extensions block");
@@ -274,24 +279,28 @@ std::unique_ptr<X509_Certificate_Data> parse_x509_cert_body(const X509_Object& o
    // Check for self-signed vs self-issued certificates
    if(data->m_subject_dn == data->m_issuer_dn)
       {
-      data->m_self_signed = false;
-
-      try
+      if(data->m_subject_key_id.empty() == false && data->m_authority_key_id.empty() == false)
          {
-         std::unique_ptr<Public_Key> pub_key(
-            X509::load_key(ASN1::put_in_sequence(data->m_subject_public_key_bits)));
-
-         Certificate_Status_Code sig_status = obj.verify_signature(*pub_key);
-
-         if(sig_status == Certificate_Status_Code::OK ||
-            sig_status == Certificate_Status_Code::SIGNATURE_ALGO_UNKNOWN)
-            {
-            data->m_self_signed = true;
-            }
+         data->m_self_signed = (data->m_subject_key_id == data->m_authority_key_id);
          }
-      catch(...)
+      else
          {
-         // ignore errors here to allow parsing to continue
+         try
+            {
+            std::unique_ptr<Public_Key> pub_key(X509::load_key(data->m_subject_public_key_bits_seq));
+
+            Certificate_Status_Code sig_status = obj.verify_signature(*pub_key);
+
+            if(sig_status == Certificate_Status_Code::OK ||
+               sig_status == Certificate_Status_Code::SIGNATURE_ALGO_UNKNOWN)
+               {
+               data->m_self_signed = true;
+               }
+            }
+         catch(...)
+            {
+            // ignore errors here to allow parsing to continue
+            }
          }
       }
 
@@ -373,6 +382,11 @@ const std::vector<uint8_t>& X509_Certificate::subject_public_key_bits() const
    return data().m_subject_public_key_bits;
    }
 
+const std::vector<uint8_t>& X509_Certificate::subject_public_key_info() const
+   {
+   return data().m_subject_public_key_bits_seq;
+   }
+
 const std::vector<uint8_t>& X509_Certificate::subject_public_key_bitstring() const
    {
    return data().m_subject_public_key_bitstring;
@@ -574,6 +588,9 @@ const AlternativeName& X509_Certificate::issuer_alt_name() const
 std::vector<std::string>
 X509_Certificate::subject_info(const std::string& req) const
    {
+   if(req == "Email")
+      return this->subject_info("RFC822");
+
    if(subject_dn().has_field(req))
       return subject_dn().get_attribute(req);
 
@@ -630,7 +647,7 @@ std::unique_ptr<Public_Key> X509_Certificate::load_subject_public_key() const
    {
    try
       {
-      return std::unique_ptr<Public_Key>(X509::load_key(ASN1::put_in_sequence(this->subject_public_key_bits())));
+      return std::unique_ptr<Public_Key>(X509::load_key(subject_public_key_info()));
       }
    catch(std::exception& e)
       {
