@@ -79,7 +79,10 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache, ro
 
     if target in ['bsi', 'nist']:
         # Arbitrarily test disable static on module policy builds
-        flags += ['--module-policy=%s' % (target), '--disable-static']
+        # x509 is optional for bsi/nist but add it so verify tests work with these minimized configs
+        flags += ['--module-policy=%s' % (target),
+                  '--enable-modules=x509',
+                  '--disable-static']
 
     if target == 'docs':
         flags += ['--with-doxygen', '--with-sphinx', '--with-rst2man']
@@ -128,7 +131,7 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache, ro
             if target == 'cross-arm32':
                 flags += ['--cpu=armv7', '--cc-abi-flags=-arch armv7 -arch armv7s -stdlib=libc++']
             elif target == 'cross-arm64':
-                flags += ['--cpu=armv8-a', '--cc-abi-flags=-arch arm64 -stdlib=libc++']
+                flags += ['--cpu=arm64', '--cc-abi-flags=-arch arm64 -stdlib=libc++']
             else:
                 raise Exception("Unknown cross target '%s' for iOS" % (target))
         elif target == 'cross-win32':
@@ -145,7 +148,7 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache, ro
                 cc_bin = 'arm-linux-gnueabihf-g++'
                 test_prefix = ['qemu-arm', '-L', '/usr/arm-linux-gnueabihf/']
             elif target == 'cross-arm64':
-                flags += ['--cpu=armv8-a']
+                flags += ['--cpu=aarch64']
                 cc_bin = 'aarch64-linux-gnu-g++'
                 test_prefix = ['qemu-aarch64', '-L', '/usr/aarch64-linux-gnu/']
             elif target == 'cross-ppc32':
@@ -167,6 +170,8 @@ def determine_flags(target, target_os, target_cpu, target_cc, cc_bin, ccache, ro
         if target_os == 'osx':
             # Test Boost on OS X
             flags += ['--with-boost']
+            # Travis has 10.12 as default image
+            flags += ['--with-os-features=getentropy']
         elif target_os == 'linux':
             flags += ['--with-lzma']
 
@@ -208,6 +213,7 @@ def run_cmd(cmd, root_dir):
     cmd = [os.path.expandvars(elem) for elem in cmd]
     sub_env = os.environ.copy()
     sub_env['LD_LIBRARY_PATH'] = root_dir
+    sub_env['PYTHONPATH'] = os.path.join(root_dir, 'src/python')
 
     redirect_stdout = None
     if len(cmd) > 3 and cmd[-2] == '>':
@@ -223,7 +229,9 @@ def run_cmd(cmd, root_dir):
 
     if proc.returncode != 0:
         print("Command failed with error code %d" % (proc.returncode))
-        sys.exit(proc.returncode)
+
+        if cmd[0] not in ['lcov']:
+            sys.exit(proc.returncode)
 
 def parse_args(args):
     """
@@ -371,6 +379,8 @@ def main(args=None):
             'src/scripts/build_docs.py',
             'src/scripts/website.py',
             'src/scripts/bench.py',
+            'src/scripts/test_python.py',
+            'src/scripts/test_cli.py',
             'src/scripts/python_unittests.py',
             'src/scripts/python_unittests_unix.py']
 
@@ -425,20 +435,22 @@ def main(args=None):
                          os.path.join(root_dir, 'fuzzer_corpus'),
                          os.path.join(root_dir, 'build/fuzzer')])
 
-        if target in ['static', 'shared'] and options.os != 'windows':
+        if target in ['shared', 'coverage'] and options.os != 'windows':
             botan_exe = os.path.join(root_dir, 'botan-cli.exe' if options.os == 'windows' else 'botan')
-            cmds.append([py_interp,
-                         os.path.join(root_dir, 'src/scripts/cli_tests.py'),
-                         botan_exe])
 
-        botan_py = os.path.join(root_dir, 'src/python/botan2.py')
+            test_scripts = ['cli_tests.py', 'test_cli.py']
+            for script in test_scripts:
+                cmds.append([py_interp, os.path.join(root_dir, 'src/scripts', script),
+                             botan_exe])
+
+        python_tests = os.path.join(root_dir, 'src/scripts/test_python.py')
 
         if target in ['shared', 'coverage']:
 
             if use_python2:
-                cmds.append(['python2', botan_py])
+                cmds.append(['python2', '-b', python_tests])
             if use_python3:
-                cmds.append(['python3', botan_py])
+                cmds.append(['python3', '-b', python_tests])
 
         if target in ['shared', 'static', 'bsi', 'nist']:
             cmds.append(make_cmd + ['install'])
@@ -449,7 +461,7 @@ def main(args=None):
             cmds.append(['llvm-cov', 'show', './botan-test',
                          '-instr-profile=botan.profdata',
                          '>', 'build/cov_report.txt'])
-            sonar_config = os.path.join(root_dir, os.path.join(root_dir, 'src/build-data/sonar-project.properties'))
+            sonar_config = os.path.join(root_dir, os.path.join(root_dir, 'src/configs/sonar-project.properties'))
             cmds.append(['sonar-scanner',
                          '-Dproject.settings=%s' % (sonar_config),
                          '-Dsonar.login=$SONAR_TOKEN'])
@@ -475,7 +487,7 @@ def main(args=None):
             if have_prog('coverage'):
                 cmds.append(['coverage', 'run', '--branch',
                              '--rcfile', os.path.join(root_dir, 'src/configs/coverage.rc'),
-                             botan_py])
+                             python_tests])
 
             if have_prog('codecov'):
                 # If codecov exists assume we are on Travis and report to codecov.io
