@@ -199,9 +199,10 @@ class Filter_Tests final : public Test
 
          Botan::Pipe pipe;
 
-         pipe.append_filter(nullptr); // ignored
          pipe.append(nullptr); // ignored
+         pipe.append_filter(nullptr); // ignored
          pipe.prepend(nullptr); // ignored
+         pipe.prepend_filter(nullptr); // ignored
          pipe.pop(); // empty pipe, so ignored
 
          std::unique_ptr<Botan::Filter> queue_filter(new Botan::SecureQueue);
@@ -218,6 +219,9 @@ class Filter_Tests final : public Test
          pipe.append_filter(new Botan::BitBucket); // succeeds
          pipe.pop();
 
+         pipe.prepend_filter(new Botan::BitBucket); // succeeds
+         pipe.pop();
+
          pipe.start_msg();
 
          std::unique_ptr<Botan::Filter> filter(new Botan::BitBucket);
@@ -228,6 +232,10 @@ class Filter_Tests final : public Test
                             "Cannot call Pipe::append_filter after start_msg",
                             [&]() { pipe.append_filter(filter.get()); });
 
+         result.test_throws("pipe error",
+                            "Cannot call Pipe::prepend_filter after start_msg",
+                            [&]() { pipe.prepend_filter(filter.get()); });
+
          result.test_throws("pipe error",
                             "Cannot append to a Pipe while it is processing",
                             [&]() { pipe.append(filter.get()); });
@@ -246,6 +254,10 @@ class Filter_Tests final : public Test
                             "Cannot call Pipe::append_filter after start_msg",
                             [&]() { pipe.append_filter(filter.get()); });
 
+         result.test_throws("pipe error",
+                            "Cannot call Pipe::prepend_filter after start_msg",
+                            [&]() { pipe.prepend_filter(filter.get()); });
+
          result.test_throws("pipe error",
                             "Invalid argument Pipe::read: Invalid message number 100",
                             [&]() { uint8_t b; size_t got = pipe.read(&b, 1, 100); BOTAN_UNUSED(got); });
@@ -263,7 +275,16 @@ class Filter_Tests final : public Test
 
 #if defined(BOTAN_HAS_CODEC_FILTERS) && defined(BOTAN_HAS_HMAC) && defined(BOTAN_HAS_SHA2_32)
          const Botan::SymmetricKey key("FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF");
-         Botan::Pipe pipe(new Botan::MAC_Filter("HMAC(SHA-256)", key, 12),
+
+         Botan::Keyed_Filter* mac_filter = new Botan::MAC_Filter("HMAC(SHA-256)", key, 12);
+
+         mac_filter->set_iv(Botan::InitializationVector()); // ignored
+
+         result.test_throws("Keyed_Filter::set_iv throws if not implemented",
+                            "Invalid argument IV length 1 is invalid for HMAC(SHA-256)",
+                            [mac_filter]() { mac_filter->set_iv(Botan::InitializationVector("AA")); });
+
+         Botan::Pipe pipe(mac_filter,
                           new Botan::Base64_Encoder);
 
          pipe.process_msg("Hi");
@@ -402,7 +423,7 @@ class Filter_Tests final : public Test
 
 #if defined(BOTAN_HAS_AES) && defined(BOTAN_HAS_MODE_CBC) && defined(BOTAN_HAS_CIPHER_MODE_PADDING)
          Botan::Cipher_Mode_Filter* cipher =
-            new Botan::Cipher_Mode_Filter(Botan::get_cipher_mode("AES-128/CBC/PKCS7", Botan::ENCRYPTION));
+            new Botan::Cipher_Mode_Filter(Botan::Cipher_Mode::create("AES-128/CBC/PKCS7", Botan::ENCRYPTION));
 
          result.test_eq("Cipher filter name", cipher->name(), "AES-128/CBC/PKCS7");
 
@@ -437,7 +458,7 @@ class Filter_Tests final : public Test
          result.test_eq("Ciphertext3", ciphertext3, "1241B9976F73051BCF809525D6E86C25");
 
          Botan::Cipher_Mode_Filter* dec_cipher =
-            new Botan::Cipher_Mode_Filter(Botan::get_cipher_mode("AES-128/CBC/PKCS7", Botan::DECRYPTION));
+            new Botan::Cipher_Mode_Filter(Botan::Cipher_Mode::create("AES-128/CBC/PKCS7", Botan::DECRYPTION));
          pipe.append(dec_cipher);
          dec_cipher->set_key(Botan::SymmetricKey("AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"));
          dec_cipher->set_iv(Botan::InitializationVector("AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAB"));
@@ -654,10 +675,20 @@ class Filter_Tests final : public Test
          Test::Result result("Filter Chain");
 
 #if defined(BOTAN_HAS_CODEC_FILTERS) && defined(BOTAN_HAS_SHA2_32) && defined(BOTAN_HAS_SHA2_64)
+
+         Botan::Filter* filters[2] = {
+            new Botan::Hash_Filter("SHA-256"),
+            new Botan::Hex_Encoder
+         };
+
+         std::unique_ptr<Botan::Chain> chain(new Botan::Chain(filters, 2));
+
+         result.test_eq("Chain has a name", chain->name(), "Chain");
+
          std::unique_ptr<Botan::Fork> fork(
             new Botan::Fork(
-               new Botan::Chain(new Botan::Hash_Filter("SHA-256"), new Botan::Hex_Encoder),
-               new Botan::Chain(new Botan::Hash_Filter("SHA-512-256"), new Botan::Hex_Encoder)
+               chain.release(),
+               new Botan::Chain(new Botan::Hash_Filter("SHA-512-256", 19), new Botan::Hex_Encoder)
             ));
 
          result.test_eq("Fork has a name", fork->name(), "Fork");
@@ -670,7 +701,7 @@ class Filter_Tests final : public Test
          result.test_eq("Hash 1", pipe.read_all_as_string(0),
                         "C00862D1C6C1CF7C1B49388306E7B3C1BB79D8D6EC978B41035B556DBB3797DF");
          result.test_eq("Hash 2", pipe.read_all_as_string(1),
-                        "610480FFA82F24F6926544B976FE387878E3D973C03DFD591C2E9896EFB903E0");
+                        "610480FFA82F24F6926544B976FE387878E3D9");
 #endif
 
          return result;
@@ -709,7 +740,7 @@ class Filter_Tests final : public Test
          {
          Test::Result result("Threaded_Fork");
 
-#if defined(BOTAN_TARGET_OS_HAS_THREADS) && defined(BOTAN_HAS_CODEC_FILTERS) && defined(BOTAN_HAS_SHA2_32)
+#if defined(BOTAN_HAS_THREAD_UTILS) && defined(BOTAN_HAS_CODEC_FILTERS) && defined(BOTAN_HAS_SHA2_32)
          Botan::Pipe pipe(new Botan::Threaded_Fork(new Botan::Hex_Encoder, new Botan::Base64_Encoder));
 
          result.test_eq("Message count", pipe.message_count(), 0);
